@@ -32,4 +32,10 @@ ALTER EXTENSION pg_hint_plan UPDATE TO "1.4.2";
 \dx+ pg_hint_plan
 ALTER EXTENSION pg_hint_plan UPDATE TO "1.4.3";
 \dx+ pg_hint_plan
+ALTER EXTENSION pg_hint_plan UPDATE TO "1.5";
+\dx+ pg_hint_plan
+ALTER EXTENSION pg_hint_plan UPDATE TO "1.5.1";
+\dx+ pg_hint_plan
+ALTER EXTENSION pg_hint_plan UPDATE TO "1.5.2";
+\dx+ pg_hint_plan
 DROP EXTENSION pg_hint_plan;
